@@ -158,7 +158,7 @@ class collection(object):
                     index[c]=index[c]+1 
                 if index[c]==nitems-1:
                     c=c-1
-	return False
+        return False
 
     def exist(self,f):
         if not self.items or not inspect.getargspec(f)[0]:
@@ -188,12 +188,11 @@ class collection(object):
                     index[c]=index[c]+1 
                 if index[c]==nitems-1:
                     c=c-1
-	return False
+        return False
 
 
     def moveUp(self, value):
-        """
-        Move element up. Owner is notified about the change.
+        """Move element up. Owner is notified about the change.
         """
         i1 = self.items.index(value)
         i2 = i1 - 1
@@ -205,8 +204,7 @@ class collection(object):
 
 
     def moveDown(self, value):
-        """
-        Move element down. Owner is notified about the change.
+        """Move element down. Owner is notified about the change.
         """
         i1 = self.items.index(value)
         i2 = i1 + 1
