@@ -108,8 +108,10 @@ def parse_attribute(self, s):
             self.upperValue.value = None
         if self.defaultValue:
             self.defaultValue.value = None
-        if self.taggedValue:
-            self.taggedValue.value = None
+        #if self.taggedValue:
+        #    self.taggedValue.value = None
+        while self.taggedValue:
+            self.taggedValue[0].unlink()
     else:
         from uml2 import LiteralSpecification
         create = self._factory.create
@@ -128,9 +130,17 @@ def parse_attribute(self, s):
         if not self.defaultValue:
             self.defaultValue = create(LiteralSpecification)
         self.defaultValue.value = g('default')
-        if not self.taggedValue:
-            self.taggedValue = create(LiteralSpecification)
-        self.taggedValue.value = g('tags')
+        #if not self.taggedValue:
+        #    self.taggedValue = create(LiteralSpecification)
+        #self.taggedValue.value = g('tags')
+        while self.taggedValue:
+            self.taggedValue[0].unlink()
+        tags = g('tags')
+        if tags:
+            for t in map(str.strip, tags.split(',')):
+                tv = create(LiteralSpecification)
+                tv.value = t
+                self.taggedValue = tv
         #print g('vis'), g('derived'), g('name'), g('type'), g('mult_l'), g('mult_u'), g('default'), g('tags')
 
 def parse_association_end(self, s):
@@ -150,9 +160,17 @@ def parse_association_end(self, s):
         if not self.upperValue:
             self.upperValue = create(LiteralSpecification)
         self.upperValue.value = g('mult_u')
-        if not self.taggedValue:
-            self.taggedValue = create(LiteralSpecification)
-        self.taggedValue.value = g('tags')
+        #if not self.taggedValue:
+        #    self.taggedValue = create(LiteralSpecification)
+        #self.taggedValue.value = g('tags')
+        while self.taggedValue:
+            self.taggedValue[0].unlink()
+        tags = g('tags')
+        if tags:
+            for t in map(str.strip, tags.split(',')):
+                tv = create(LiteralSpecification)
+                tv.value = t
+                self.taggedValue = tv
         # We have multiplicity
     else:
         #print 'namematch'
@@ -177,10 +195,18 @@ def parse_association_end(self, s):
                 if not self.upperValue:
                     self.upperValue = create(LiteralSpecification)
                 self.upperValue.value = g('mult_u')
-            if g('tags'):
-                if not self.taggedValue:
-                    self.taggedValue = create(LiteralSpecification)
-                self.taggedValue.value = g('tags')
+            #if g('tags'):
+            #    if not self.taggedValue:
+            #        self.taggedValue = create(LiteralSpecification)
+            #    self.taggedValue.value = g('tags')
+            tags = g('tags')
+            if tags:
+                while self.taggedValue:
+                    self.taggedValue[0].unlink()
+                for t in map(str.strip, tags.split(',')):
+                    tv = create(LiteralSpecification)
+                    tv.value = t
+                    self.taggedValue = tv
 
 def parse_property(self, s):
     if self.association:
@@ -218,10 +244,17 @@ def parse_operation(self, s):
             p.upperValue = create(LiteralSpecification)
         p.upperValue.value = g('mult_u')
         # FIXME: Maybe add to Operation.ownedRule?
-        if not p.taggedValue:
-            p.taggedValue = create(LiteralSpecification)
-        p.taggedValue.value = g('tags')
-        #print g('vis'), g('name'), g('type'), g('mult_l'), g('mult_u'), g('tags')
+        #if not p.taggedValue:
+        #    p.taggedValue = create(LiteralSpecification)
+        #p.taggedValue.value = g('tags')
+        while self.taggedValue:
+            self.taggedValue[0].unlink()
+        tags = g('tags')
+        if tags:
+            for t in map(str.strip, tags.split(',')):
+                tv = create(LiteralSpecification)
+                tv.value = t
+                p.taggedValue = tv
         
         pindex = 0
         params = g('params')
@@ -248,9 +281,16 @@ def parse_operation(self, s):
             if not p.defaultValue:
                 p.defaultValue = create(LiteralSpecification)
             p.defaultValue.value = g('default')
-            if not p.taggedValue:
-                p.taggedValue = create(LiteralSpecification)
-            p.taggedValue.value = g('tags')
+            #if not p.taggedValue:
+            #    p.taggedValue = create(LiteralSpecification)
+            #p.taggedValue.value = g('tags')
+            while self.taggedValue:
+                self.taggedValue[0].unlink()
+            if tags:
+                for t in map(str.strip, tags.split(',')):
+                    tv = create(LiteralSpecification)
+                    tv.value = t
+                    p.taggedValue = tv
             self.formalParameter = p
 
             #print ' ', g('dir') or 'in', g('name'), g('type'), g('mult_l'), g('mult_u'), g('default'), g('tags')
@@ -337,8 +377,13 @@ def render_attribute(self, visibility=False, is_derived=False, type=False,
     if default and self.defaultValue and self.defaultValue.value:
         s.write(' = %s' % self.defaultValue.value)
 
-    if tags and self.taggedValue and self.taggedValue.value:
-        s.write(' { %s }' % self.taggedValue.value)
+    #if tags and self.taggedValue and self.taggedValue.value:
+    #    s.write(' { %s }' % self.taggedValue.value)
+    if self.taggedValue:
+        tvs = ', '.join(filter(None, map(getattr, self.taggedValue,
+                                          ['value'] * len(self.taggedValue))))
+        if tvs:
+            s.write(' { %s }' % tvs)
     s.reset()
     return s.read()
 
@@ -364,8 +409,13 @@ def render_association_end(self):
             m.write('%s..%s' % (self.lowerValue.value, self.upperValue.value))
         else:
             m.write('%s' % self.upperValue.value)
-    if self.taggedValue and self.taggedValue.value:
-        m.write(' {%s}' % self.taggedValue.value)
+    #if self.taggedValue and self.taggedValue.value:
+    #    m.write(' {%s}' % self.taggedValue.value)
+    if self.taggedValue:
+        tvs = ',\n '.join(filter(None, map(getattr, self.taggedValue,
+                                     ['value'] * len(self.taggedValue))))
+        if tvs:
+            m.write(' { %s }' % tvs)
     m.reset()
     mult = m.read()
 
@@ -421,8 +471,12 @@ def render_operation(self, visibility=False, type=False, multiplicity=False,
                 s.write('[%s]' % p.upperValue.value)
         if default and p.defaultValue and p.defaultValue.value:
             s.write(' = %s' % p.defaultValue.value)
-        if tags and  p.taggedValue and p.taggedValue.value:
-            s.write(' { %s }' % p.taggedValue.value)
+        #if tags and  p.taggedValue and p.taggedValue.value:
+        #    s.write(' { %s }' % p.taggedValue.value)
+        if p.taggedValue:
+             tvs = ', '.join(filter(None, map(getattr, p.taggedValue,
+                                              ['value'] * len(p.taggedValue))))
+             s.write(' { %s }' % tvs)
         if p is not self.formalParameter[-1]:
             s.write(', ')
 
@@ -437,8 +491,13 @@ def render_operation(self, visibility=False, type=False, multiplicity=False,
                 s.write('[%s..%s]' % (rr.lowerValue.value, rr.upperValue.value))
             else:
                 s.write('[%s]' % rr.upperValue.value)
-        if tags and rr.taggedValue and rr.taggedValue.value:
-            s.write(' { %s }' % rr.taggedValue.value)
+        #if tags and rr.taggedValue and rr.taggedValue.value:
+        #    s.write(' { %s }' % rr.taggedValue.value)
+        if rr.taggedValue:
+            tvs = ', '.join(filter(None, map(getattr, rr.taggedValue,
+                                             ['value'] * len(rr.taggedValue))))
+            if tvs:
+                s.write(' { %s }' % tvs)
     s.reset()
     return s.read()
 
