@@ -5,15 +5,16 @@ Generalization --
 
 import gobject
 import diacanvas
-import gaphor
-import gaphor.UML as UML
-from gaphor.diagram import initialize_item
-import relationship
 
-class GeneralizationItem(relationship.RelationshipItem):
+from gaphor import resource
+from gaphor import UML
+from gaphor.diagram import initialize_item
+from gaphor.diagram.relationship import RelationshipItem
+
+class GeneralizationItem(RelationshipItem):
 
     def __init__(self, id=None):
-        relationship.RelationshipItem.__init__(self, id)
+        RelationshipItem.__init__(self, id)
         self.set(has_head=1, head_fill_color=0,
                  head_a=15.0, head_b=15.0, head_c=10.0, head_d=10.0)
         
@@ -76,7 +77,7 @@ class GeneralizationItem(relationship.RelationshipItem):
             s2 = c2.subject
             relation = self.find_relationship(s1, s2)
             if not relation:
-                relation = gaphor.resource(UML.ElementFactory).create(UML.Generalization)
+                relation = resource(UML.ElementFactory).create(UML.Generalization)
                 relation.general = s1
                 relation.specific = s2
             self.subject = relation
