@@ -39,7 +39,9 @@ class GIdleThread(object):
     for nasty locking schemes.
 
     Example (runs a counter through the GLib main loop routine):
-    >>> def counter(max): for x in xrange(max): yield x
+    >>> def counter(max):
+    ...     for x in xrange(max):
+    ...         yield x
     >>> t = GIdleThread(counter(123))
     >>> t.start()
     >>> while gen.is_alive():
