@@ -14,6 +14,9 @@ from gaphor.ui.abstractwindow import AbstractWindow
 import gaphor.ui.diagramactions
 import gaphor.diagram.placementactions
 
+from gaphor.event import DiagramItemFocused
+from zope import component
+
 class DiagramTab(object):
     
     def __init__(self, owning_window):
@@ -43,6 +46,11 @@ class DiagramTab(object):
             # Set capabilities:
             #self.__on_diagram_undo(diagram.canvas.undo_manager)
 
+            if hasattr(self, 'view'):
+                self.view.get_hadjustment().set_value(0.0)
+                self.view.get_vadjustment().set_value(0.0)
+
+
     def construct(self):
         title = self.diagram and self.diagram.name or '<None>'
 
@@ -81,6 +89,9 @@ class DiagramTab(object):
         item_tool = ItemTool(self.owning_window.get_action_pool())
         view.get_default_tool().set_item_tool(item_tool)
 
+        view.get_hadjustment().set_value(0.0)
+        view.get_vadjustment().set_value(0.0)
+
         table.show_all()
 
         self.owning_window.new_tab(self, table, title)
@@ -126,6 +137,7 @@ class DiagramTab(object):
 
     def __on_view_focus_item(self, view, focus_item):
         self.owning_window.execute_action('ItemFocus')
+        component.handle(DiagramItemFocused(focus_item))
 
     def __on_view_select_item(self, view, select_item):
         self.owning_window.execute_action('ItemSelect')
