@@ -53,7 +53,7 @@ def show_status_window(title, message, parent=None, queue=None):
         except QueueEmpty:
             pass
         if percentage:
-            progress_bar.set_fraction(percentage / 100.0)
+            progress_bar.set_fraction(min(percentage / 100.0, 100.0))
         return True
 
     if queue:
@@ -109,6 +109,7 @@ class NewAction(Action):
 weave_method(NewAction.execute, ErrorHandlerAspect, message='Could not create a new model.')
 register_action(NewAction)
 
+
 class RevertAction(Action):
     id = 'FileRevert'
     label = _('_Revert...')
@@ -314,10 +315,11 @@ class QuitAction(Action):
         self._window = window
 
     def execute(self):
-        log.debug('Quiting gaphor...')
-        self._window.close()
-        del self._window
-        gc.collect()
+        if self._window.ask_to_close():
+            log.debug('Quiting gaphor...')
+            self._window.close()
+            del self._window
+            gc.collect()
 
 register_action(QuitAction)
 
@@ -362,7 +364,7 @@ register_action(OpenConsoleWindowAction)
 class ManualAction(Action):
     id = 'Manual'
     label = _('_Manual')
-    stock_id = 'gtk-about'
+    stock_id = 'gtk-help'
     tooltip='Manual for Gaphor'
 
     def init(self, window):
@@ -418,7 +420,7 @@ class AboutAction(Action):
         #add_label('<span size="xx-large" weight="bold">Gaphor</span>')
         add_label('<span weight="bold">version %s</span>' % version)
         add_label('<span variant="smallcaps">UML Modeling tool for GNOME</span>', 8, 8)
-        add_label('<span size="small">Copyright (c) 2001-2004 Arjan J. Molenaar</span>', 8, 8)
+        add_label('<span size="small">Copyright (c) 2001-2006 Arjan J. Molenaar</span>', 8, 8)
         #vbox.pack_start(gtk.HSeparator())
         notebook.append_page(tab_vbox, gtk.Label('About'))
 
