@@ -15,6 +15,11 @@ from gaphor.ui.menufactory import toolbox_to_menu
 
 # Load actions
 from gaphor.ui import mainactions, diagramactions
+from gaphor.ui.objectinspector import ObjectInspector
+
+
+from gaphor.interfaces import *
+from zope import component
 
 class MainWindow(AbstractWindow):
     """The main window for the application.
@@ -36,13 +41,19 @@ class MainWindow(AbstractWindow):
                 'InsertImplementation')),
         (_('Components'), (
                 'InsertComponent',
+                'InsertAssemblyConnector',
                 'InsertNode',
                 'InsertArtifact')),
+#        (_('Composite Structures'), (
+#                'InsertConnector',)),
         (_('Actions'), (
                 'InsertAction',
                 'InsertInitialNode',
                 'InsertActivityFinalNode',
+                'InsertFlowFinalNode',
                 'InsertDecisionNode',
+                'InsertForkNode',
+                'InsertObjectNode',
                 'InsertFlow')),
 #        (_('Interactions'), (
 #                'InsertInteraction',
@@ -118,7 +129,7 @@ class MainWindow(AbstractWindow):
             _('_Help'), (
                 'Manual',
                 'About',
-                '<HelpSlot>')
+                '<HelpSlot>'),
             )
 
     toolbar =  ('FileOpen',
@@ -204,6 +215,18 @@ class MainWindow(AbstractWindow):
         tab = self.get_current_diagram_tab()
         return tab and tab.get_view()
 
+    def ask_to_close(self):
+        """
+        Ask user to close window.
+        """
+        dialog = gtk.MessageDialog(self.get_window(),
+            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
+            gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
+            _("Quit Gaphor?"))
+        answer = dialog.run()
+        dialog.destroy()
+        return answer == gtk.RESPONSE_YES
+
     def show_diagram(self, diagram):
         """Show a Diagram element in a new tab. If a tab is already open,
         show that one instead.
@@ -239,16 +262,34 @@ class MainWindow(AbstractWindow):
         paned = gtk.HPaned()
         paned.set_property('position', 160)
         paned.pack1(vbox)
+        
         notebook = gtk.Notebook()
         #notebook.popup_enable()
         notebook.set_scrollable(True)
         notebook.set_show_border(False)
- 
+        #notebook.set_size_request(-1, 10000)
+
         notebook.connect_after('switch-page', self.on_notebook_switch_page)
 
-        paned.pack2(notebook)
+        self.objectInspector = ObjectInspector()
+        #self.objectInspector.set_size_request(-1, 50)
+
+        diagramReceivedFocus = component.adapter(IDiagramElementReceivedFocus)(
+            self.objectInspector)
+        component.provideHandler(diagramReceivedFocus)
+        
+        secondPaned = gtk.VPaned()
+        secondPaned.set_property('position',
+                                 int(resource('ui.object-inspector-position', 600)))
+        secondPaned.pack1(notebook)
+        secondPaned.pack2(self.objectInspector)
+        secondPaned.show_all()
+        paned.pack2(secondPaned)
         paned.show_all()
 
+        secondPaned.connect('notify::position',
+                            self.on_object_inspector_notify_position)
+
         self.notebook = notebook
         self.model = model
         self.view = view
@@ -259,6 +300,8 @@ class MainWindow(AbstractWindow):
                                size=window_size,
                                contents=paned)
 
+        self.window.connect('delete-event', self._on_window_delete)
+
         vbox.set_border_width(3)
 
         toolbox = Toolbox(self.menu_factory, self.toolbox)
@@ -356,6 +399,7 @@ class MainWindow(AbstractWindow):
         a Diagram).
         """
         path = self.get_model().path_from_element(element)
+        #log.debug("PATH = %s" % path)
         # Expand the first row:
         self.get_tree_view().expand_row(path[:-1], False)
         # Select the diagram, so it can be opened by the OpenModelElement action
@@ -372,6 +416,9 @@ class MainWindow(AbstractWindow):
         del self.model
         del self.view
 
+    def _on_window_delete(self, window = None, event = None):
+        return not self.ask_to_close()
+
     def on_view_event(self, view, event):
         """Show a popup menu if button3 was pressed on the TreeView.
         """
@@ -401,6 +448,10 @@ class MainWindow(AbstractWindow):
     def on_window_size_allocate(self, window, allocation):
         resource.set('ui.window-size', (allocation.width, allocation.height), persistent=True)
 
+    def on_object_inspector_notify_position(self, paned, arg):
+        resource.set('ui.object-inspector-position',
+                     paned.get_position(), persistent=True)
+
 #    def on_toolbox_toggled(self, toolbox, box_name, visible):
 #        print 'Box', box_name, 'is visible:', visible
 #        #resource.set('ui.toolbox.%s' % box_name, visible, persistent=True)
