@@ -8,7 +8,8 @@ TODO: show tooltips in the status bar when a menu item is selected.
 import gobject
 import gtk
 from gaphor.misc.action import ActionError, Action, CheckAction, RadioAction
-from gaphor.misc.action import get_actions_for_slot
+from gaphor.misc.action import SlotMenu
+#from gaphor.misc.action import get_actions_for_slot
 import gaphor.ui.wrapbox
 
 __all__ = [ 'MenuFactory' ]
@@ -51,17 +52,19 @@ class MenuFactory(object):
     instance has been created.
 
     NOTE: When you use GtkWidget.show_all() to show your window, also
-    invisible menu items will be shown. 
+    invisible menu items will be shown (this is not desirable). 
     """
 
-    def __init__(self, action_pool, accel_group=None,
+    def __init__(self, window_class, action_pool, accel_group=None,
                  statusbar=None, statusbar_context=0):
         self.action_pool = action_pool
         self.in_activation = False
         
         self.accel_group = accel_group
+        self.accel_class = window_class + 'Action'
         self.statusbar = statusbar
         self.statusbar_context = statusbar_context
+        #self._slot_to_menu_item = {}
 
     def get_action(self, action_id):
         """Find the action, create a new one if not found.
@@ -81,16 +84,21 @@ class MenuFactory(object):
             if type(id) in (tuple, list):
                 # Create and populate a new submenu
                 submenu = gtk.Menu()
+                submenu.set_accel_group(self.accel_group)
                 self._create_menu_items(id, groups, submenu)
                 item.set_submenu(submenu)
+                submenu.owning_menu_item = item
+                self._set_submenu_sensitivity(submenu)
                 submenu.show()
             elif id.startswith('<') and id.endswith('>'):
                 # We're dealing with a placeholder here, strip the <>
-                slot_def = get_actions_for_slot(id[1:-1])
-                self._create_menu_items(slot_def, groups, menu, item)
+                #slot_def = get_actions_for_slot(id[1:-1])
+                #self._create_menu_items(slot_def, groups, menu, item)
+                self.create_slot(id[1:-1], menu)
             else:
                 item = self.create_menu_item(id, groups)
                 menu.add(item)
+                self.set_accelerator(id, item)
 
     def create_menu(self, menu_def):
         """Create a menu bar.
@@ -161,7 +169,6 @@ class MenuFactory(object):
                 item.connect('activate', self.on_radio_item_activate, action.id)
                 self._connect_item_to_action('notify::active', action, item,
                                             self.on_radio_menu_item_notify_active)
-
             elif isinstance(action, CheckAction):
                 item = gtk.CheckMenuItem(label)
                 item.set_active(action.active)
@@ -170,7 +177,7 @@ class MenuFactory(object):
                                             self.on_menu_item_notify_action)
             else:
                 if action.stock_id:
-                    item = gtk.ImageMenuItem(action.stock_id, self.accel_group)
+                    item = gtk.ImageMenuItem(action.stock_id)
                     # stock_info: (id, label, mod, key, translationdomain)
                     # Set the label right
                     item.get_children()[0].set_text_with_mnemonic(label)
@@ -178,20 +185,10 @@ class MenuFactory(object):
                     item = gtk.MenuItem(label)
                 item.connect('activate', self.on_item_activate, action.id)
 
-            # If possible, set a accelarator (and the accelerator is not yet
-            # set by the stock id):
-            if action.accel and not (stock_info and stock_info[3]):
-                modifier, keyval = _mod_and_keyval_from_accel(action.accel)
-                #print 'adding accel', action.accel, 'to', action.id, gtk.accelerator_name(modifier, keyval)
-                item.add_accelerator("activate", self.accel_group,
-                                     keyval, modifier, gtk.ACCEL_VISIBLE);
-
             # Connect to the event so we can push/pop status messages:
             item.show()
             item.connect('event', self.on_item_event, action.id)
             item.set_property('visible', action.visible)
-            #if not action.visible: item.hide()
-            #print 'menu item: visible =', item.get_property('visible')
             item.set_property('sensitive', action.sensitive)
             self._connect_item_to_action('notify::visible', action, item,
                                         self.on_menu_item_notify_action)
@@ -199,6 +196,57 @@ class MenuFactory(object):
                                         self.on_menu_item_notify_action)
         return item
 
+    def set_accelerator(self, id, item):
+        """When available, set an accelerator in the action. This
+        is done by creating an accel_path on the item (<window-class>/action.id)
+        and defining the accelerator in the global gtk.AccelMap.
+        """
+        action = self.get_action(id)
+        if not action:
+            return
+
+        keyval = None
+        modifier = None
+
+        #  Fetch accelerator from the stock id.
+        if action.stock_id:
+            stock_info = gtk.stock_lookup(action.stock_id)
+            # stock_info : (id, label, mod, key, translationdomain)
+            try:
+                keyval = stock_info[3]
+                modifier = stock_info[2]
+            except TypeError:
+                pass
+
+        # Use an accelerator defined in the action class.
+        if not keyval and action.accel:
+            modifier, keyval = _mod_and_keyval_from_accel(action.accel)
+
+        # No accelerator at all.
+        if not keyval:
+            return
+
+        accel_path = '<' + self.accel_class + '>/' + id
+        item.set_accel_path(accel_path)
+        gtk.accel_map_add_entry(accel_path, keyval, modifier)
+            
+    def create_slot(self, slot_id, menu):
+        """Create a slot. A slot can change it's content during the lifetime
+        of the application. A handler is connected to the slot's 'rebuild'
+        signal, so the menu is rebuild when the content has changed.
+        """
+        slot_menu = self.action_pool.get_slot(slot_id)
+        # Create a menu item that acts as a placeholder:
+        item = gtk.MenuItem()
+        item.slot_size = 0
+        menu.add(item)
+        #self._slot_to_menu_item[slot_menu] = item
+        #self._connect_item_to_action('rebuild')
+        handler = slot_menu.connect('rebuild', self.on_slot_menu_rebuild, item)
+        item.connect('destroy', self.on_slot_menu_destroy, slot_menu, handler)
+
+        self.on_slot_menu_rebuild(slot_menu, item)
+
     def create_toolbar(self, menu_def):
         """Create a Toolbar. items in menu_def should not be nested as
         they are when creating a normal menu.
@@ -223,6 +271,7 @@ class MenuFactory(object):
                         group = groups[action.group]
                     except KeyError:
                         group = None
+                    # TODO: replace with toolbar.insert(item, index (-1=append))
                     item = toolbar.append_element(gtk.TOOLBAR_CHILD_RADIOBUTTON,
                                     None, label, action.tooltip, '', icon,
                                     callback=self.on_radio_item_activate,
@@ -255,8 +304,7 @@ class MenuFactory(object):
                 item.set_property('sensitive', action.sensitive)
                 self._connect_item_to_action('notify::visible', action, item,
                                              self.on_menu_item_notify_action)
-                self._connect_item_to_action('notify::sensitive', action, item,
-                                             self.on_menu_item_notify_action)
+                self._connect_item_to_action('notify::sensitive', action, item, self.on_menu_item_notify_action)
         return toolbar
 
     def create_button(self, id, groups=None, tooltips=None):
@@ -307,6 +355,7 @@ class MenuFactory(object):
                     #item.set_property('use-stock', True)
                 #else:
                 item = gtk.Button()
+
             if action.stock_id:
                 #item.set_property('stock-id', action.stock_id)
                 item.add(icon)
@@ -349,8 +398,42 @@ class MenuFactory(object):
         return wrapbox
 
     def on_item_activate(self, menu_item, action_id):
+        """Call the action that is associated with the menu item.
+        """
         self.action_pool.execute(action_id)
 
+    def on_slot_menu_rebuild(self, slot_menu, slot_item):
+        """Rebuild the content of a slot menu in a menu. This is done
+        by finding a specific placeholder in the menu and removing all
+        contained elements. After that the elements are created again
+        based on the action id's obtained from DynamicMenu.get_menu().
+        """
+        #log.debug('Rebuilding slot %s (%s)' % (slot_menu.slot_id, type(slot_menu)))
+        #slot_item = self._slot_to_menu_item[slot_menu]
+        menu = slot_item.get_parent()
+        children = menu.get_children()
+        slot_index = children.index(slot_item) + 1
+
+        # Remove the old menu items:
+        if slot_item.slot_size > 0:
+            for i in xrange(slot_index, slot_index + slot_item.slot_size):
+                children[i].destroy()
+
+        slot_item.slot_size = 0
+
+        # Now create the new items. First create them in a list, then insert
+        # them into the menu at the right place.
+        groups = {}
+        menu_list = FakeMenu()
+        self._create_menu_items(slot_menu.get_menu(), groups, menu_list)
+
+        # Add them to the menu
+        for item in menu_list:
+            menu.insert(item, slot_index)
+            slot_index += 1
+            slot_item.slot_size += 1
+        self._set_submenu_sensitivity(menu)
+
     def on_menu_item_notify_action(self, menu_item, pspec, action_id):
         """Helper callback, copies state from the action to the menu item.
         """
@@ -401,6 +484,9 @@ class MenuFactory(object):
         action = self.get_action(action_id)
         action.disconnect(handler)
 
+    def on_slot_menu_destroy(self, menu_item, slot_menu, handler):
+        slot_menu.disconnect(handler)
+
     def _connect_item_to_action(self, signal_name, action, item, handler):
         """Connect the items signal handler to the action. The handler is
         destroyed when the item is destroyed.
@@ -424,6 +510,19 @@ class MenuFactory(object):
             icon = None
         return icon, label
 
+    def _set_submenu_sensitivity(self, menu):
+        """Determine if a menu entry containing a submenu should
+        be sensitive. If the entry contains no (visible) menu items,
+        the menu item is made insensitive.
+        """
+        if not hasattr(menu, 'owning_menu_item'):
+            return
+
+        if [ c for c in menu.get_children() if c.get_property('visible')]:
+            menu.owning_menu_item.set_property('sensitive', True)
+        else:
+            menu.owning_menu_item.set_property('sensitive', False)
+
 
 def toolbox_to_menu(toolbox):
     """Given a toolbox definition, create a list suitable for
@@ -438,3 +537,15 @@ def toolbox_to_menu(toolbox):
             for item in box[1]:
                 l.append(item)
     return l
+
+
+class FakeMenu(object):
+
+    def __init__(self):
+        self.l = []
+
+    def add(self, item):
+        self.l.append(item)
+
+    def __iter__(self):
+        return self.l.__iter__()
