@@ -6,12 +6,13 @@ a result only classifiers are shown here.
 
 import gobject
 import gtk
+import operator
 import stock
 
-import gaphor.UML as UML
+from gaphor import UML
 from gaphor import resource
 from gaphor.undomanager import get_undo_manager
-import operator
+
 
 # The following items will not be shown in the treeview, although they
 # are UML.Namespace elements.
@@ -54,7 +55,11 @@ class NamespaceModel(gtk.GenericTreeModel):
         path = self.path_from_element(element)
         #print 'new_node_from_element', path, element, element.name
         self.row_inserted(path, self.get_iter(path))
+
         element.connect('name', self.on_name_changed)
+        if isinstance(element, UML.Classifier) \
+                or isinstance(element, UML.Operation):
+            element.connect('isAbstract', self.on_element_changed)
 
         if isinstance(element, UML.Namespace):
             element.connect('ownedMember', self.on_ownedmember_changed)
@@ -65,9 +70,16 @@ class NamespaceModel(gtk.GenericTreeModel):
     def detach_notifiers_from_node(self, node):
         """Detach notifiers for node
         """
-        node[0].disconnect(self.on_name_changed)
-        if isinstance(node[0], UML.Namespace):
-            node[0].disconnect(self.on_ownedmember_changed)
+        element = node[0]
+
+        element.disconnect(self.on_name_changed)
+
+        if isinstance(element, UML.Classifier) \
+                or isinstance(element, UML.Operation):
+            element.disconnect(self.on_element_changed)
+
+        if isinstance(element, UML.Namespace):
+            element.disconnect(self.on_ownedmember_changed)
             for child in node[1]:
                 self.detach_notifiers_from_node(child)
 
@@ -164,6 +176,13 @@ class NamespaceModel(gtk.GenericTreeModel):
                 doit(child, depth + 1)
         doit(self.root)
 
+    def on_element_changed(self, element, pspec):
+        """element changed, update appropriate row
+        """
+        path = self.path_from_element(element)
+        if path:
+            self.row_changed(path, self.get_iter(path))
+
     def on_name_changed(self, element, pspec):
         """the name of element has changed, update the tree model
         """
@@ -345,6 +364,7 @@ class NamespaceModel(gtk.GenericTreeModel):
         #print "on_iter_parent", node
         return self.node_from_element(node[0].namespace)
 
+
 class NamespaceView(gtk.TreeView):
 
     TARGET_STRING = 0
@@ -366,7 +386,7 @@ class NamespaceView(gtk.TreeView):
         self.icon_cache = {}
 
         self.set_property('headers-visible', 0)
-        self.set_rules_hint(gtk.TRUE)
+        self.set_rules_hint(True)
         selection = self.get_selection()
         selection.set_mode(gtk.SELECTION_BROWSE)
         column = gtk.TreeViewColumn ('')
@@ -378,9 +398,9 @@ class NamespaceView(gtk.TreeView):
         # Second cell if for the name of the object...
         cell = gtk.CellRendererText ()
         #cell.set_property ('editable', 1)
-        cell.connect('edited', self._name_edited)
+        cell.connect('edited', self._text_edited)
         column.pack_start (cell, 0)
-        column.set_cell_data_func (cell, self._set_name, None)
+        column.set_cell_data_func (cell, self._set_text, None)
 
         assert len (column.get_cell_renderers()) == 2
         self.append_column (column)
@@ -424,13 +444,24 @@ class NamespaceView(gtk.TreeView):
             self.icon_cache[type(value)] = icon
         cell.set_property('pixbuf', icon)
 
-    def _set_name (self, column, cell, model, iter, data):
+
+    def _set_text (self, column, cell, model, iter, data):
+        """set font and of model elements in tree view
+        """
         value = model.get_value(iter, 0)
         #print 'set_name:', value
-        name = value and (value.name or '').replace('\n', ' ') or '<None>'
-        cell.set_property('text', name)
+        text = value and (value.name or '').replace('\n', ' ') or '<None>'
 
-    def _name_edited(self, cell, path_str, new_text):
+        if isinstance(value, UML.Diagram):
+            text = '<b>%s</b>' % text
+        elif (isinstance(value, UML.Classifier) 
+                or isinstance(value, UML.Operation)) and value.isAbstract:
+            text = '<i>%s</i>' % text
+
+        cell.set_property('markup', text)
+
+
+    def _text_edited(self, cell, path_str, new_text):
         """The text has been edited. This method updates the data object.
         Note that 'path_str' is a string where the fields are separated by
         colons ':', like this: '0:1:1'. We first turn them into a tuple.
@@ -439,7 +470,9 @@ class NamespaceView(gtk.TreeView):
             model = self.get_property('model')
             iter = model.get_iter_from_string(path_str)
             element = model.get_value(iter, 0)
+            get_undo_manager().begin_transaction()
             element.name = new_text
+            get_undo_manager().commit_transaction()
         except Exception, e:
             log.error('Could not create path from string "%s"' % path_str)
 
@@ -504,9 +537,9 @@ class NamespaceView(gtk.TreeView):
 
             except AttributeError:
                 #print dir(context)
-                context.drop_finish(gtk.FALSE, time)
+                context.drop_finish(False, time)
             else:
-                context.drop_finish(gtk.TRUE, time)
+                context.drop_finish(True, time)
                 # Finally let's try to select the element again.
                 path = model.path_from_element(element)
                 self.expand_row(path[:-1], False)
@@ -527,3 +560,40 @@ gobject.type_register(NamespaceView)
 #if __debug__:
 #    from gaphor.misc.aspects import weave_method, TimerAspect
 #    NamespaceModel.sort_node = weave_method(NamespaceModel.sort_node, TimerAspect)
+
+
+#class NamespaceViewRow(object):
+#
+#    def __init__(self, context):
+#        self.context = context
+#
+#    def get_icon_id(self):
+#        """Return the icon that matches the row."""
+#        return stock.get_stock_id(type(self.context))
+#
+#    def get_text(self):
+#        """Return the text to be set on the row."""
+#        return self.context and (self.context.name or '').replace('\n', ' ') or '<None>'
+#
+#    def set_text(self, new_text):
+#        """Set a new text that was entered while editing the row line."""
+#        self.context.name = new_text
+#
+#registerAdapterFactory(UML.Namespace, NamespaceViewRow, NamespaceViewRow)
+
+
+#class PropertyViewRow(NamespaceViewRow):
+#
+#    def get_text(self):
+#        context = self.context
+#        if context:
+#            name = context.name
+#            type = context.typeValue and context.typeValue.value or ''
+#            if type:
+#                return name + ': ' + type
+#            else:
+#                return name
+#        else:
+#            return '<None>'
+#
+#registerAdapterFactory(UML.Property, NamespaceViewRow, PropertyViewRow)
