@@ -1,4 +1,4 @@
-#vim:sw=4:et
+# vim:sw=4:et
 
 import pygtk
 pygtk.require('2.0')
@@ -7,12 +7,19 @@ import gtk
 
 
 class Wrapbox(gtk.Table):
+    """A Wrapbox contains a set of items. A wrap box tries to optimize it's
+    content by moving elements to a second row if the do not fit on the first.
+    And a third and a fourth, depending on the given space.
+
+    The width is given, the height is changed in order to fit all contained
+    objects.
+    """
 
     def __init__(self):
         self.__gobject_init__()
         self.resize_idle_id = 0
         self.rows = 1
-        self.cols = 0
+        self.cols = 1
         #self.table = gtk.Table(self.rows, self.cols)
         self.resize(self.rows, self.cols)
         #self.table.connect('size_allocate', self.on_size_allocate)
